/************************************************************************/
/*																		*/
/*	R2R.cpp		--		Definition for R2R library 	    				*/
/*																		*/
/************************************************************************/
/*	Author:		Eric Marsh												*/
/*	Copyright 2016, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*		This file defines functions for the PmodDA4						*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	06/30/2016(EricM): created											*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include "R2R.h"
#include <Dspi.h>


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*        R2R::R2R
**
**        Synopsis:
**				
**        Parameters:
**
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**			Class constructor. Performs variables initialization tasks
**
**
*/
R2R::R2R()
{
	pdspi = NULL;
	
}

/* ------------------------------------------------------------ */
/*        R2R::begin
**
**        Synopsis:
**				myR2R.begin(PAR_ACCESS_SPI0);
**        Parameters:
**				uint8_t bAccessType	- the SPI interface where the Pmod is connected. It can be one of:
**					0	PAR_ACCESS_SPI0	- indicates SPI port 2, connetor JB
**					1	PAR_ACCESS_SPI1	- indicates SPI port 1, connector J1
**
**				uint8_t spd - The frequency at which the SPI clock should be set to. 1 Mhz by default
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				This function initializes the specific SPI interface used, setting the SPI frequency to a default value of 1 MHz.
**				
**
**
*/
void R2R::begin(uint8_t bAccessType, uint32_t spd, uint8_t SSpin)
{
	Serial.begin(9600);
	if(bAccessType == PAR_ACCESS_SPI0)
	{
		pdspi = new DSPI0();
		m_SSPin = PIN_DSPI0_SS;	// default SS pin for SPI0
	}
	if(bAccessType == PAR_ACCESS_SPI1)
	{
		pdspi = new DSPI1();
		m_SSPin = PIN_DSPI1_SS;	// default SS pin for SPI1
	}
	if(pdspi != NULL)
	{
		// if SSpin was passed in as a parameter, set m_SSPin to it.
		if( SSpin != -1)
		{
			m_SSPin = SSpin;
		}
		pdspi->begin(m_SSPin);
		pdspi->setMode(DSPI_MODE0);
		pdspi->setSpeed(spd);
	}
	
	pinTable = {m_SSPin, m_SSPin + 1, m_SSPin + 2, m_SSPin + 3, 
		m_SSPin + 4, m_SSPin + 5, m_SSPin + 6, m_SSPin + 7};
	
	for(int i = 0; i < 8; i++)
	{
		pinMode(pinTable[i], OUTPUT);
	}
}

/* ------------------------------------------------------------ */
/*        R2R::ConvertIntegerToBinaryArray
**
**        Synopsis:
**				myR2R.ConvertIntegerToBinaryArray(15);
**        Parameters:
**				int num - the integer to convert into the int array, binArray, defined in R2R.h
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				Converts the integer passed in the parameter, converts it into a binary array, 
**				and stores it in the predefined int array, binArray
**
**
*/
void R2R::ConvertIntegerToBinaryArray(int num)
{
	int mask = 1 << (7);
	for(int x = 7; x >= 0; x--)
	{
		if( (num & mask) == 0 )
		{
			binArray[x] = 0;
		}
		else
		{
			binArray[x] = 1;
		}
	mask  >>= 1;
   }
   return;
}

/* ------------------------------------------------------------ */
/*        R2R::WriteBinaryArray
**
**        Synopsis:
**				myR2R.WriteBinaryArray(binArray);
**        Parameters:
**				int* binArray - an integer array, containing a binary representation of an integer
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				Reads each element in binArray, and tells its respective pin # to output low or high voltage.
**				The result is sending the binary integer to the VOUT output.
**
**
*/
void R2R::SendBinaryArray(int *binArray)
{
	for(int j = 0; j < 8; j++)
    {
		digitalWrite(pinTable[j], binArray[j]);
    }
}
/* ------------------------------------------------------------ */
/*        R2R::SendInteger
**
**        Synopsis:
**				myR2R.SendInteger(15);
**        Parameters:
**				int num - the number to be converted into binary and sent to VOUT
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				Convert integer to binary, and send that value to VOUT output.
**
**
*/
void R2R::SendInteger(int num)
{
	// convert num to binary array and store it into predefined int array, binArray
	ConvertIntegerToBinaryArray(num);
	
	// send binnary array to analog output 
	SendBinaryArray(binArray);
}

/* ------------------------------------------------------------ */
/*        R2R::PrintBinaryArray
**
**        Synopsis:
**				myR2R.PrintBinaryArray(15);
**        Parameters:
**				None
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				Prints the binary value that is sent to the VOUT analog output. This value is stored
**				in the predefined int array, binArray[8], before it is sent off.
**
**
*/
void R2R::PrintBinaryArray()
{
	for(int i = 7; i >= 0; i--)
	{
		Serial.print(binArray[i]);
	}
	Serial.print("\n");
}